@@ -5,12 +5,13 @@ from django.urls import URLPattern, URLResolver
 from rest_framework import views, viewsets
 from rest_framework.schemas.generators import BaseSchemaGenerator  # type: ignore
 from rest_framework.schemas.generators import EndpointEnumerator as BaseEndpointEnumerator
+from rest_framework.test import APIRequestFactory
 
 from drf_spectacular.extensions import OpenApiViewExtension
 from drf_spectacular.plumbing import (
-    ComponentRegistry, alpha_operation_sorter, build_root_object, error, is_versioning_supported,
-    modify_for_versioning, normalize_result_object, operation_matches_version,
-    reset_generator_stats, warn,
+    ComponentRegistry, alpha_operation_sorter, build_root_object, camelize_operation, error,
+    is_versioning_supported, modify_for_versioning, normalize_result_object,
+    operation_matches_version, reset_generator_stats, sanitize_result_object, warn,
 )
 from drf_spectacular.settings import spectacular_settings
 
@@ -127,6 +128,14 @@ class SchemaGenerator(BaseSchemaGenerator):
             if not self.has_view_permissions(path, method, view):
                 continue
 
+            # mocked request to allow certain operations in get_queryset and get_serializer[_class]
+            # without exceptions being raised due to no request.
+            if not request:
+                request = getattr(APIRequestFactory(), method.lower())(path=path)
+                request = view.initialize_request(request)
+
+            view.request = request
+
             if view.versioning_class and not is_versioning_supported(view.versioning_class):
                 warn(
                     f'using unsupported versioning class "{view.versioning_class}". view will be '
@@ -154,6 +163,9 @@ class SchemaGenerator(BaseSchemaGenerator):
                 path = path[1:]
             path = urljoin(self.url or '/', path)
 
+            if spectacular_settings.CAMELIZE_NAMES:
+                path, operation = camelize_operation(path, operation)
+
             result.setdefault(path, {})
             result[path][method.lower()] = operation
 
@@ -168,4 +180,5 @@ class SchemaGenerator(BaseSchemaGenerator):
         )
         for hook in spectacular_settings.POSTPROCESSING_HOOKS:
             result = hook(result=result, generator=self, request=request, public=public)
-        return normalize_result_object(result)
+
+        return sanitize_result_object(normalize_result_object(result))
