@@ -1,6 +1,7 @@
 import hashlib
 import inspect
 import json
+import re
 import sys
 from abc import ABCMeta
 from collections import OrderedDict, defaultdict
@@ -9,10 +10,14 @@ from decimal import Decimal
 from enum import Enum
 from typing import DefaultDict, Generic, List, Optional, Type, TypeVar, Union
 
+import inflection
 import uritemplate
 from django import __version__ as DJANGO_VERSION
 from django.apps import apps
-from django.urls.resolvers import _PATH_PARAMETER_COMPONENT_RE, get_resolver  # type: ignore
+from django.urls.resolvers import (  # type: ignore
+    _PATH_PARAMETER_COMPONENT_RE, RegexPattern, Resolver404, RoutePattern, URLPattern, URLResolver,
+    get_resolver,
+)
 from django.utils.functional import Promise
 from django.utils.module_loading import import_string
 from rest_framework import exceptions, fields, mixins, serializers, versioning
@@ -112,9 +117,11 @@ def is_field(obj):
     return isinstance(force_instance(obj), fields.Field) and not is_serializer(obj)
 
 
-def is_basic_type(obj):
+def is_basic_type(obj, allow_none=True):
     if not isinstance(obj, Hashable):
         return False
+    if not allow_none and (obj is None or obj is OpenApiTypes.NONE):
+        return False
     return obj in OPENAPI_TYPE_MAPPING or obj in PYTHON_TYPE_MAPPING
 
 
@@ -146,6 +153,27 @@ def get_lib_doc_excludes():
     ]
 
 
+def get_view_model(view):
+    """
+    obtain model from view via view's queryset. try safer view attribute first
+    before going through get_queryset(), which may perform arbitrary operations.
+    """
+    model = getattr(getattr(view, 'queryset', None), 'model', None)
+
+    if model is not None:
+        return model
+
+    try:
+        return view.get_queryset().model
+    except Exception as exc:
+        warn(
+            f'failed to obtain model through view\'s queryset due to raised exception. '
+            f'prevent this either by setting "queryset = Model.objects.none()" on the view, '
+            f'having an empty fallback in get_queryset() or by using @extend_schema. '
+            f'(Exception: {exc})'
+        )
+
+
 def get_doc(obj):
     """ get doc string with fallback on obj's base classes (ignoring DRF documentation). """
     if not inspect.isclass(obj):
@@ -342,7 +370,7 @@ def _follow_field_source(model, path):
             if not target_model:
                 raise UnableToProceedError(
                     f'could not follow field source through intermediate property "{path[0]}" '
-                    f'on model {model}. please add a type hint on the model\'s property/function'
+                    f'on model {model}. please add a type hint on the model\'s property/function '
                     f'to enable traversal of the source path "{".".join(path)}".'
                 )
             return _follow_field_source(target_model, path[1:])
@@ -560,7 +588,7 @@ def load_enum_name_overrides():
 
 
 def list_hash(lst):
-    return hashlib.blake2b(json.dumps(list(lst), sort_keys=True).encode()).hexdigest()
+    return hashlib.sha256(json.dumps(list(lst), sort_keys=True).encode()).hexdigest()
 
 
 def resolve_regex_path_parameter(path_regex, variable, available_formats):
@@ -593,10 +621,11 @@ def resolve_regex_path_parameter(path_regex, variable, available_formats):
 
 
 def is_versioning_supported(versioning_class):
-    return bool(
-        issubclass(versioning_class, versioning.URLPathVersioning)
-        or issubclass(versioning_class, versioning.NamespaceVersioning)
-    )
+    return issubclass(versioning_class, (
+        versioning.URLPathVersioning,
+        versioning.NamespaceVersioning,
+        versioning.AcceptHeaderVersioning
+    ))
 
 
 def operation_matches_version(view, requested_version):
@@ -609,13 +638,9 @@ def operation_matches_version(view, requested_version):
 
 
 def modify_for_versioning(patterns, method, path, view, requested_version):
-    assert view.versioning_class
+    assert view.versioning_class and view.request
 
-    from rest_framework.test import APIRequestFactory
-    mocked_request = getattr(APIRequestFactory(), method.lower())(path=path)
-    view.request = mocked_request
-
-    mocked_request.version = requested_version
+    view.request.version = requested_version
 
     if issubclass(view.versioning_class, versioning.URLPathVersioning):
         version_param = view.versioning_class.version_param
@@ -626,11 +651,60 @@ def modify_for_versioning(patterns, method, path, view, requested_version):
         # emulate router behaviour by injecting substituted variable into view
         view.kwargs[version_param] = requested_version
     elif issubclass(view.versioning_class, versioning.NamespaceVersioning):
-        mocked_request.resolver_match = get_resolver(tuple(patterns)).resolve(path)
+        try:
+            view.request.resolver_match = get_resolver(
+                urlconf=tuple(detype_pattern(p) for p in patterns)
+            ).resolve(path)
+        except Resolver404:
+            error(f"namespace versioning path resolution failed for {path}. path will be ignored.")
+    elif issubclass(view.versioning_class, versioning.AcceptHeaderVersioning):
+        renderer = view.get_renderers()[0]
+        view.request.META['HTTP_ACCEPT'] = f'{renderer.media_type}; version={requested_version}'
+
+        negotiated = view.perform_content_negotiation(view.request)
+        view.request.accepted_renderer, view.request.accepted_media_type = negotiated
 
     return path
 
 
+def detype_pattern(pattern):
+    """
+    return an equivalent pattern that accepts arbitrary values for path parameters.
+    de-typing the path will ease determining a matching route without having properly
+    formatted dummy values for all path parameters.
+    """
+    if isinstance(pattern, URLResolver):
+        return URLResolver(
+            pattern=detype_pattern(pattern.pattern),
+            urlconf_name=[detype_pattern(p) for p in pattern.urlconf_name],
+            default_kwargs=pattern.default_kwargs,
+            app_name=pattern.app_name,
+            namespace=pattern.namespace,
+        )
+    elif isinstance(pattern, URLPattern):
+        return URLPattern(
+            pattern=detype_pattern(pattern.pattern),
+            callback=pattern.callback,
+            default_args=pattern.default_args,
+            name=pattern.name,
+        )
+    elif isinstance(pattern, RoutePattern):
+        return RoutePattern(
+            route=re.sub(r'<\w+:(\w+)>', r'<\1>', pattern._route),
+            name=pattern.name,
+            is_endpoint=pattern._is_endpoint
+        )
+    elif isinstance(pattern, RegexPattern):
+        return RegexPattern(
+            regex=re.sub(r'\(\?P<(\w+)>.*\)', r'(?P<\1>[^/]+)', pattern._regex),
+            name=pattern.name,
+            is_endpoint=pattern._is_endpoint
+        )
+    else:
+        warn(f'unexpected pattern "{pattern}" encountered while simplifying urlpatterns.')
+        return pattern
+
+
 def normalize_result_object(result):
     """ resolve non-serializable objects like lazy translation strings and OrderedDict """
     if isinstance(result, dict) or isinstance(result, OrderedDict):
@@ -640,3 +714,36 @@ def normalize_result_object(result):
     if isinstance(result, Promise):
         return str(result)
     return result
+
+
+def sanitize_result_object(result):
+    # warn about and resolve operationId collisions with suffixes
+    operations = defaultdict(list)
+    for path, methods in result['paths'].items():
+        for method, operation in methods.items():
+            operations[operation['operationId']].append((path, method))
+    for operation_id, paths in operations.items():
+        if len(paths) == 1:
+            continue
+        warn(f'operationId "{operation_id}" has collisions {paths}. resolving with numeral suffixes.')
+        for idx, (path, method) in enumerate(sorted(paths)[1:], start=2):
+            suffix = str(idx) if spectacular_settings.CAMELIZE_NAMES else f'_{idx}'
+            result['paths'][path][method]['operationId'] += suffix
+
+    return result
+
+
+def camelize_operation(path, operation):
+    for path_variable in re.findall(r'\{(\w+)\}', path):
+        path = path.replace(
+            f'{{{path_variable}}}',
+            f'{{{inflection.camelize(path_variable, False)}}}'
+        )
+
+    for parameter in operation.get('parameters', []):
+        if parameter['in'] == OpenApiParameter.PATH:
+            parameter['name'] = inflection.camelize(parameter['name'], False)
+
+    operation['operationId'] = inflection.camelize(operation['operationId'], False)
+
+    return path, operation
