@@ -1,32 +1,41 @@
 from collections import namedtuple
-from typing import Dict, Any
+from typing import Any, Dict
 
+from django.conf import settings
+from django.utils import translation
+from django.utils.translation import gettext_lazy as _
 from rest_framework.renderers import TemplateHTMLRenderer
 from rest_framework.response import Response
 from rest_framework.reverse import reverse
 from rest_framework.views import APIView
 
 from drf_spectacular.renderers import (
-    OpenApiJsonRenderer, OpenApiJsonRenderer2,
-    OpenApiYamlRenderer, OpenApiYamlRenderer2
+    OpenApiJsonRenderer, OpenApiJsonRenderer2, OpenApiYamlRenderer, OpenApiYamlRenderer2,
 )
 from drf_spectacular.settings import spectacular_settings
 from drf_spectacular.types import OpenApiTypes
-from drf_spectacular.utils import extend_schema
+from drf_spectacular.utils import OpenApiParameter, extend_schema
 
 if spectacular_settings.SERVE_INCLUDE_SCHEMA:
     SCHEMA_KWARGS: Dict[str, Any] = {'responses': {200: OpenApiTypes.OBJECT}}
+
+    if settings.USE_I18N:
+        SCHEMA_KWARGS['parameters'] = [
+            OpenApiParameter(
+                'lang', str, OpenApiParameter.QUERY, enum=list(dict(settings.LANGUAGES).keys())
+            )
+        ]
 else:
     SCHEMA_KWARGS = {'exclude': True}
 
 
 class SpectacularAPIView(APIView):
-    """
+    __doc__ = _("""
     OpenApi3 schema for this API. Format can be selected via content negotiation.
 
     - YAML: application/vnd.oai.openapi
     - JSON: application/vnd.oai.openapi+json
-    """
+    """)
     renderer_classes = [
         OpenApiYamlRenderer, OpenApiYamlRenderer2, OpenApiJsonRenderer, OpenApiJsonRenderer2
     ]
@@ -43,9 +52,15 @@ class SpectacularAPIView(APIView):
             ModuleWrapper = namedtuple('ModuleWrapper', ['urlpatterns'])
             self.urlconf = ModuleWrapper(tuple(self.urlconf))
 
+        if settings.USE_I18N and request.GET.get('lang'):
+            with translation.override(request.GET.get('lang')):
+                return self._get_schema_response(request)
+        else:
+            return self._get_schema_response(request)
+
+    def _get_schema_response(self, request):
         generator = self.generator_class(urlconf=self.urlconf, api_version=self.api_version)
-        schema = generator.get_schema(request=request, public=self.serve_public)
-        return Response(schema)
+        return Response(generator.get_schema(request=request, public=self.serve_public))
 
 
 class SpectacularYAMLAPIView(SpectacularAPIView):
@@ -60,12 +75,16 @@ class SpectacularSwaggerView(APIView):
     renderer_classes = [TemplateHTMLRenderer]
     permission_classes = spectacular_settings.SERVE_PERMISSIONS
     url_name = 'schema'
+    url = None
     template_name = 'drf_spectacular/swagger_ui.html'
 
     @extend_schema(exclude=True)
     def get(self, request, *args, **kwargs):
+        schema_url = self.url or reverse(self.url_name, request=request)
+        if request.GET.get('lang'):
+            schema_url += f'{"&" if "?" in schema_url else "?"}lang={request.GET.get("lang")}'
         return Response(
-            {'url_name': reverse(self.url_name, request=request)},
+            {'schema_url': schema_url},
             template_name=self.template_name
         )
 
@@ -74,11 +93,15 @@ class SpectacularRedocView(APIView):
     renderer_classes = [TemplateHTMLRenderer]
     permission_classes = spectacular_settings.SERVE_PERMISSIONS
     url_name = 'schema'
+    url = None
     template_name = 'drf_spectacular/redoc.html'
 
     @extend_schema(exclude=True)
     def get(self, request, *args, **kwargs):
+        schema_url = self.url or reverse(self.url_name, request=request)
+        if request.GET.get('lang'):
+            schema_url += f'{"&" if "?" in schema_url else "?"}lang={request.GET.get("lang")}'
         return Response(
-            {'url_name': reverse(self.url_name, request=request)},
+            {'schema_url': schema_url},
             template_name=self.template_name
         )
