@@ -3,9 +3,9 @@ import yaml
 from django.conf.urls import include
 from django.db import models
 from django.urls import path, re_path
-from rest_framework import mixins, routers, serializers, viewsets
+from rest_framework import generics, mixins, routers, serializers, viewsets
 from rest_framework.test import APIClient
-from rest_framework.versioning import NamespaceVersioning, URLPathVersioning
+from rest_framework.versioning import AcceptHeaderVersioning, NamespaceVersioning, URLPathVersioning
 
 from drf_spectacular.generators import SchemaGenerator
 from drf_spectacular.utils import extend_schema
@@ -45,6 +45,10 @@ class NamespaceVersioningViewset(PathVersioningViewset):
     versioning_class = NamespaceVersioning
 
 
+class AcceptHeaderVersioningViewset(PathVersioningViewset):
+    versioning_class = AcceptHeaderVersioning
+
+
 class PathVersioningViewset2(mixins.ListModelMixin, viewsets.GenericViewSet):
     versioning_class = URLPathVersioning
     queryset = VersioningModel.objects.all()
@@ -65,6 +69,10 @@ class NamespaceVersioningViewset2(PathVersioningViewset2):
     versioning_class = NamespaceVersioning
 
 
+class AcceptHeaderVersioningViewset2(PathVersioningViewset2):
+    versioning_class = AcceptHeaderVersioning
+
+
 @pytest.mark.parametrize('viewset_cls', [PathVersioningViewset, PathVersioningViewset2])
 @pytest.mark.parametrize('version', ['v1', 'v2'])
 def test_url_path_versioning(no_warnings, viewset_cls, version):
@@ -94,6 +102,45 @@ def test_namespace_versioning(no_warnings, viewset_cls, version):
     assert_schema(schema, f'tests/test_versioning_{version}.yml')
 
 
+def test_namespace_versioning_urlpatterns_simplification(no_warnings):
+    class NamespaceVersioningAPIView(generics.RetrieveUpdateDestroyAPIView):
+        versioning_class = NamespaceVersioning
+        serializer_class = Xv1Serializer
+        queryset = VersioningModel.objects.all()
+
+    urls = (
+        path('x/<int:pk>/', NamespaceVersioningAPIView.as_view()),
+        path('y/<pk>/', NamespaceVersioningAPIView.as_view()),
+        re_path('z/(?P<pk>[0-9A-Fa-f-]+)/', NamespaceVersioningAPIView.as_view()),
+    )
+    generator = SchemaGenerator(
+        patterns=[path('v1/<int:some_param>/', include((urls, 'v1'))), ],
+        api_version='v1',
+    )
+    schema = generator.get_schema(request=None, public=True)
+
+    for s in ['x', 'y', 'z']:
+        parameters = schema['paths'][f'/v1/{{some_param}}/{s}/{{id}}/']['get']['parameters']
+        parameters = {p['name']: p for p in parameters}
+        assert parameters['id']['schema']['type'] == 'integer'
+        assert parameters['some_param']['schema']['type'] == 'integer'
+
+
+@pytest.mark.parametrize('viewset_cls', [AcceptHeaderVersioningViewset, AcceptHeaderVersioningViewset2])
+@pytest.mark.parametrize('version', ['v1', 'v2'])
+def test_accept_header_versioning(no_warnings, viewset_cls, version):
+    router = routers.SimpleRouter()
+    router.register('x', viewset_cls, basename='x')
+    generator = SchemaGenerator(
+        patterns=[
+            path('', include((router.urls, 'x'))),
+        ],
+        api_version=version,
+    )
+    schema = generator.get_schema(request=None, public=True)
+    assert_schema(schema, f'tests/test_versioning_accept_{version}.yml')
+
+
 urlpatterns_namespace = [
     path('x/', NamespaceVersioningViewset.as_view({'get': 'list'})),
     path('schema/', SpectacularAPIView.as_view(
@@ -112,12 +159,23 @@ urlpatterns_path = [
         versioning_class=URLPathVersioning, url_name='schema-pv-versioned'
     )),
 ]
+urlpatterns_accept_header = [
+    path('x/', AcceptHeaderVersioningViewset2.as_view({'get': 'list'})),
+    path('schema/', SpectacularAPIView.as_view(
+        versioning_class=AcceptHeaderVersioning
+    ), name='schema-ahv-versioned'),
+    path('schema/ui', SpectacularSwaggerView.as_view(
+        versioning_class=AcceptHeaderVersioning, url_name='schema-ahv-versioned'
+    )),
+]
 urlpatterns = [
     # path versioning
     re_path(r'^api/pv/(?P<version>[v1|v2]+)/', include(urlpatterns_path)),
     # namespace versioning
     path('api/nv/v1/', include((urlpatterns_namespace, 'v1'))),
     path('api/nv/v2/', include((urlpatterns_namespace, 'v2'))),
+    # accept header versioning
+    path('api/ahv/', include((urlpatterns_accept_header, 'x'))),
     # all unversioned
     path('api/schema/', SpectacularAPIView.as_view()),
     # manually versioned schema view that is in itself unversioned
@@ -126,9 +184,9 @@ urlpatterns = [
 
 
 @pytest.mark.parametrize(['url', 'path_count'], [
-    ('/api/nv/v2/schema/', 6),  # v2 nv + v2 pv + unversioned
-    ('/api/pv/v1/schema/', 6),  # v1 nv + v1 pv + unversioned
-    ('/api/schema-v2/', 6),  # v2 nv + v2 pv + unversioned
+    ('/api/nv/v2/schema/', 8),  # v2 nv + v2 pv + v2 ahv + unversioned
+    ('/api/pv/v1/schema/', 8),  # v1 nv + v1 pv + v1 ahv + unversioned
+    ('/api/schema-v2/', 8),  # v2 nv + v2 pv + v2 ahv + unversioned
     ('/api/schema/', 2),  # unversioned schema
 ])
 @pytest.mark.urls(__name__)
@@ -140,6 +198,16 @@ def test_spectacular_view_versioning(no_warnings, url, path_count):
     assert len(schema['paths']) == path_count
 
 
+@pytest.mark.parametrize('version', ['v1', 'v2'])
+@pytest.mark.urls(__name__)
+def test_spectacular_view_accept_header_versioning(no_warnings, version):
+    response = APIClient().get('/api/ahv/schema/', HTTP_ACCEPT='application/json; version={version}')
+    assert response.status_code == 200
+    schema = yaml.load(response.content, Loader=yaml.SafeLoader)
+    validate_schema(schema)
+    assert len(schema['paths']) == 6
+
+
 @pytest.mark.parametrize(['url', 'schema_url'], [
     ('/api/nv/v1/schema/ui', b'/api/nv/v1/schema/'),
     ('/api/nv/v2/schema/ui', b'/api/nv/v2/schema/'),
