@@ -1,9 +1,12 @@
 from django.db import models
+from django.urls import path
 from rest_framework import mixins, serializers, views, viewsets
 from rest_framework.authentication import BaseAuthentication
-from rest_framework.decorators import action
+from rest_framework.decorators import action, api_view
 from rest_framework.views import APIView
 
+from drf_spectacular.generators import SchemaGenerator
+from drf_spectacular.types import OpenApiTypes
 from drf_spectacular.utils import extend_schema
 from tests import generate_schema
 
@@ -66,7 +69,9 @@ def test_no_queryset_warn(capsys):
         serializer_class = X1Serializer
 
     generate_schema('x1', X1Viewset)
-    assert 'no queryset' in capsys.readouterr().err
+    stderr = capsys.readouterr().err
+    assert 'obtaining queryset from' in stderr  # warning 1
+    assert 'failed to obtain model through view\'s queryset' in stderr  # warning 2
 
 
 def test_path_param_not_in_model(capsys):
@@ -146,7 +151,7 @@ def test_no_serializer_class_on_apiview(capsys):
     assert 'Unable to guess serializer for' in capsys.readouterr().err
 
 
-def test_unable_to_follow_field_source_through_intermediate_property_warning(warnings):
+def test_unable_to_follow_field_source_through_intermediate_property_warning(capsys):
     class FailingFieldSourceTraversalModel1(models.Model):
         @property
         def x(self):  # missing type hint emits warning
@@ -165,3 +170,52 @@ def test_unable_to_follow_field_source_through_intermediate_property_warning(war
             pass  # pragma: no cover
 
     generate_schema('x', view=XAPIView)
+    assert (
+        'could not follow field source through intermediate property'
+    ) in capsys.readouterr().err
+
+
+def test_unable_to_derive_function_type_warning(capsys):
+    class FailingFieldSourceTraversalModel1(models.Model):
+        @property
+        def x(self):  # missing type hint emits warning
+            return  # pragma: no cover
+
+    class XSerializer(serializers.ModelSerializer):
+        x = serializers.ReadOnlyField()
+        y = serializers.SerializerMethodField()
+
+        def get_y(self):
+            return  # pragma: no cover
+
+        class Meta:
+            model = FailingFieldSourceTraversalModel1
+            fields = '__all__'
+
+    class XAPIView(APIView):
+        @extend_schema(responses=XSerializer)
+        def get(self, request):
+            pass  # pragma: no cover
+
+    generate_schema('x', view=XAPIView)
+    stderr = capsys.readouterr().err
+    assert 'type hint for function "x" is unknown.' in stderr
+    assert 'type hint for function "get_y" is unknown.' in stderr
+
+
+def test_operation_id_collision_resolution(capsys):
+    @extend_schema(responses=OpenApiTypes.FLOAT)
+    @api_view(['GET'])
+    def view_func(request, format=None):
+        pass  # pragma: no cover
+
+    urlpatterns = [
+        path('pi/<int:foo>', view_func),
+        path('pi/', view_func),
+    ]
+    generator = SchemaGenerator(patterns=urlpatterns)
+    schema = generator.get_schema(request=None, public=True)
+
+    assert schema['paths']['/pi/']['get']['operationId'] == 'pi_retrieve'
+    assert schema['paths']['/pi/{foo}']['get']['operationId'] == 'pi_retrieve_2'
+    assert 'operationId "pi_retrieve" has collisions' in capsys.readouterr().err
